@@ -128,8 +128,8 @@ cell_unregister_span (GnmCell const * const cell)
 	if (cell->row_info->spans == NULL)
 		return;
 
-	g_hash_table_foreach_remove (cell->row_info->spans, &span_remove,
-				     (gpointer)cell);
+	g_hash_table_foreach_remove (cell->row_info->spans,
+		&span_remove, (gpointer)cell);
 }
 
 /*
@@ -168,22 +168,24 @@ row_span_get (ColRowInfo const * const ri, int const col)
  * returns TRUE if the cell is empty.
  */
 static inline gboolean
-cellspan_is_empty (int col, ColRowInfo const *ri, GnmCell const *ok_span_cell)
+cellspan_is_empty (int col, GnmCell const *ok_span_cell)
 {
-	CellSpanInfo const *span = row_span_get (ri, col);
+	CellSpanInfo const *span = row_span_get (ok_span_cell->row_info, col);
 	GnmCell const *tmp;
 
 	if (span != NULL && span->cell != ok_span_cell)
 		return FALSE;
 
-	tmp = sheet_cell_get (ok_span_cell->base.sheet, col, ri->pos);
-	/* FIXME : cannot use cell_is_empty until expressions can span.
+	tmp = sheet_cell_get (ok_span_cell->base.sheet,
+		col, ok_span_cell->pos.row);
+
+	/* FIXME : cannot use gnm_cell_is_empty until expressions can span.
 	 * because cells with expressions start out with value Empty
 	 * existing spans continue to flow through, but never get removed
 	 * because we don't respan expression results.
 	 */
 	return (tmp == NULL || tmp->value == NULL ||
-		(VALUE_IS_EMPTY (tmp->value) && !cell_has_expr(tmp)));
+		(VALUE_IS_EMPTY (tmp->value) && !gnm_cell_has_expr(tmp)));
 }
 
 /*
@@ -199,51 +201,50 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 {
 	Sheet *sheet;
 	int h_align, v_align, left, max_col, min_col;
-	int row, pos, margin;
+	int row, pos;
 	int cell_width_pixel, indented_w;
 	GnmStyle const *style;
-	ColRowInfo const *ri;
+	ColRowInfo const *ci;
 	GnmRange const *merge_left;
 	GnmRange const *merge_right;
 
 	g_return_if_fail (cell != NULL);
 
 	sheet = cell->base.sheet;
-	ri = cell->row_info;
-
-	style = cell_get_style (cell);
-	h_align = style_default_halign (style, cell);
+	style = gnm_cell_get_style (cell);
+	h_align = gnm_style_default_halign (style, cell);
 
         /*
 	 * Report only one column is used if
 	 *	- Cell is in a hidden col
-	 * 	- Cell is a number
-	 * 	- Cell is the top left of a merged cell
-	 * 	- The text fits inside column (for non center across selection)
-	 * 	- The alignment mode are set to "justify"
+	 *	- Cell is a number
+	 *	- Cell is the top left of a merged cell
+	 *	- The text fits inside column (for non center across selection)
+	 *	- The alignment mode are set to "justify"
 	 */
 	if (sheet != NULL &&
 	    h_align != HALIGN_CENTER_ACROSS_SELECTION &&
-	    (cell_is_merged (cell) ||
-	     (!sheet->display_formulas && cell_is_number (cell)))) {
+	    (gnm_cell_is_merged (cell) ||
+	     (!sheet->display_formulas && gnm_cell_is_number (cell)))) {
 		*col1 = *col2 = cell->pos.col;
 		return;
 	}
 
 	v_align = gnm_style_get_align_v (style);
 	row   = cell->pos.row;
-	indented_w = cell_width_pixel = cell_rendered_width (cell);
+	indented_w = cell_width_pixel = gnm_cell_rendered_width (cell);
 	if (h_align == HALIGN_LEFT || h_align == HALIGN_RIGHT) {
-		indented_w += cell_rendered_offset (cell);
+		indented_w += gnm_cell_rendered_offset (cell);
 		if (sheet->text_is_rtl)
 			h_align = (h_align == HALIGN_LEFT) ? HALIGN_RIGHT : HALIGN_LEFT;
 	}
 
-	if (cell_is_empty (cell) ||
-	    !cell->col_info->visible ||
+	ci = sheet_col_get_info	(sheet, cell->pos.col);
+	if (gnm_cell_is_empty (cell) ||
+	    !ci->visible ||
 	    (h_align != HALIGN_CENTER_ACROSS_SELECTION &&
 		 (gnm_style_get_wrap_text (style) ||
-		  indented_w <= COL_INTERNAL_WIDTH (cell->col_info))) ||
+		  indented_w <= COL_INTERNAL_WIDTH (ci))) ||
 	    h_align == HALIGN_JUSTIFY ||
 	    h_align == HALIGN_FILL ||
 	    h_align == HALIGN_DISTRIBUTED ||
@@ -253,7 +254,7 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 		return;
 	}
 
-	sheet_merge_get_adjacent (sheet, &cell->pos, &merge_left, &merge_right);
+	gnm_sheet_merge_get_adjacent (sheet, &cell->pos, &merge_left, &merge_right);
 	min_col = (merge_left != NULL) ? merge_left->end.col : -1;
 	max_col = (merge_right != NULL) ? merge_right->start.col : SHEET_MAX_COLS;
 
@@ -261,63 +262,55 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 	switch (h_align) {
 	case HALIGN_LEFT:
 		pos = cell->pos.col + 1;
-		left = indented_w - COL_INTERNAL_WIDTH (cell->col_info);
-		margin = cell->col_info->margin_b;
+		left = indented_w - COL_INTERNAL_WIDTH (ci);
 
 		for (; left > 0 && pos < max_col; pos++){
 			ColRowInfo const *ci = sheet_col_get_info (sheet, pos);
 
 			if (ci->visible) {
-				if (!cellspan_is_empty (pos, ri, cell))
+				if (!cellspan_is_empty (pos, cell))
 					return;
 
 				/* The space consumed is:
 				 *   - The margin_b from the last column
 				 *   - The width of the cell
 				 */
-				left -= COL_INTERNAL_WIDTH (ci) +
-					margin + ci->margin_a;
+				left -= ci->size_pixels - 1;
 				*col2 = pos;
 			}
-			margin = ci->margin_b;
 		}
 		return;
 
 	case HALIGN_RIGHT:
 		pos = cell->pos.col - 1;
-		left = indented_w - COL_INTERNAL_WIDTH (cell->col_info);
-		margin = cell->col_info->margin_a;
+		left = indented_w - COL_INTERNAL_WIDTH (ci);
 
 		for (; left > 0 && pos > min_col; pos--){
 			ColRowInfo const *ci = sheet_col_get_info (sheet, pos);
 
 			if (ci->visible) {
-				if (!cellspan_is_empty (pos, ri, cell))
+				if (!cellspan_is_empty (pos, cell))
 					return;
 
 				/* The space consumed is:
 				 *   - The margin_a from the last column
 				 *   - The width of this cell
 				 */
-				left -= COL_INTERNAL_WIDTH (ci) +
-					margin + ci->margin_b;
+				left -= ci->size_pixels - 1;
 				*col1 = pos;
 			}
-			margin = ci->margin_a;
 		}
 		return;
 
 	case HALIGN_CENTER: {
 		int remain_left, remain_right;
-		int margin_a, margin_b, pos_l, pos_r;
+		int pos_l, pos_r;
 
 		pos_l = pos_r = cell->pos.col;
-		left = cell_width_pixel -  COL_INTERNAL_WIDTH (cell->col_info);
+		left = cell_width_pixel - COL_INTERNAL_WIDTH (ci);
 
 		remain_left  = left / 2 + (left % 2);
 		remain_right = left / 2;
-		margin_a = cell->col_info->margin_a;
-		margin_b = cell->col_info->margin_b;
 
 		for (; remain_left > 0 || remain_right > 0;){
 			ColRowInfo const *ci;
@@ -326,10 +319,8 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 				ci = sheet_col_get_info (sheet, pos_l);
 
 				if (ci->visible) {
-					if (cellspan_is_empty (pos_l, ri, cell)) {
-						remain_left -= COL_INTERNAL_WIDTH (ci) +
-							margin_a + ci->margin_b;
-						margin_a = ci->margin_a;
+					if (cellspan_is_empty (pos_l, cell)) {
+						remain_left -= ci->size_pixels - 1;
 						*col1 = pos_l;
 					} else
 						remain_left = 0;
@@ -341,10 +332,8 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 				ci = sheet_col_get_info (sheet, pos_r);
 
 				if (ci->visible) {
-					if (cellspan_is_empty (pos_r, ri, cell)) {
-						remain_right -= COL_INTERNAL_WIDTH (ci) +
-							margin_b + ci->margin_a;
-						margin_b = ci->margin_b;
+					if (cellspan_is_empty (pos_r, cell)) {
+						remain_right -= ci->size_pixels - 1;
 						*col2 = pos_r;
 					} else
 						max_col = remain_right = 0;
@@ -355,16 +344,15 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 		break;
 	} /* case HALIGN_CENTER */
 
-	case HALIGN_CENTER_ACROSS_SELECTION:
-	{
-		int const row = ri->pos;
+	case HALIGN_CENTER_ACROSS_SELECTION: {
+		int const row = cell->pos.row;
 		int pos_l, pos_r;
 
 		pos_l = pos_r = cell->pos.col;
 		while (--pos_l > min_col) {
 			ColRowInfo const *ci = sheet_col_get_info (sheet, pos_l);
 			if (ci->visible) {
-				if (cellspan_is_empty (pos_l, ri, cell)) {
+				if (cellspan_is_empty (pos_l, cell)) {
 					GnmStyle const * const style =
 						sheet_style_get (cell->base.sheet, pos_l, row);
 
@@ -378,7 +366,7 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 		while (++pos_r < max_col) {
 			ColRowInfo const *ci = sheet_col_get_info (sheet, pos_r);
 			if (ci->visible) {
-				if (cellspan_is_empty (pos_r, ri, cell)) {
+				if (cellspan_is_empty (pos_r, cell)) {
 					GnmStyle const * const style =
 						sheet_style_get (cell->base.sheet, pos_r, row);
 
@@ -398,14 +386,14 @@ cell_calc_span (GnmCell const *cell, int *col1, int *col2)
 }
 
 void
-row_calc_spans (ColRowInfo *rinfo, Sheet const *sheet)
+row_calc_spans (ColRowInfo *ri, int row, Sheet const *sheet)
 {
-	int left, right, col, row = rinfo->pos;
+	int left, right, col;
 	GnmRange const *merged;
 	GnmCell *cell;
 	int const last = sheet->cols.max_used;
 
-	row_destroy_span (rinfo);
+	row_destroy_span (ri);
 	for (col = 0 ; col <= last ; ) {
 		cell = sheet_cell_get (sheet, col, row);
 		if (cell == NULL) {
@@ -420,10 +408,10 @@ row_calc_spans (ColRowInfo *rinfo, Sheet const *sheet)
 
 		/* render as necessary */
 		if (cell->rendered_value == NULL)
-			cell_render_value ((GnmCell *)cell, TRUE);
+			gnm_cell_render_value ((GnmCell *)cell, TRUE);
 
-		if (cell_is_merged (cell)) {
-			merged = sheet_merge_is_corner (sheet, &cell->pos);
+		if (gnm_cell_is_merged (cell)) {
+			merged = gnm_sheet_merge_is_corner (sheet, &cell->pos);
 			if (NULL != merged) {
 				col = merged->end.col + 1;
 				continue;
@@ -438,5 +426,5 @@ row_calc_spans (ColRowInfo *rinfo, Sheet const *sheet)
 			col++;
 	}
 
-	rinfo->needs_respan = FALSE;
+	ri->needs_respan = FALSE;
 }
