@@ -25,7 +25,6 @@
 
 #include <gsf/gsf-impl-utils.h>
 #include <gtk/gtkmain.h>
-#include <stdio.h>
 #include <stdlib.h>
 
 #define PARENT_TYPE (G_TYPE_OBJECT)
@@ -51,13 +50,11 @@ complete_finalize (GObject *object)
 		complete->idle_tag = 0;
 	}
 
-	if (complete->text) {
-		g_free (complete->text);
-		complete->text = NULL;
-	}
+	g_free (complete->text);
+	complete->text = NULL;
 
 	parent = g_type_class_peek (PARENT_TYPE);
-	(parent->finalize) (object);
+	parent->finalize (object);
 }
 
 static gint
@@ -85,8 +82,7 @@ complete_start (Complete *complete, char const *text)
 	g_return_if_fail (text != NULL);
 
 	if (complete->text != text) {
-		if (complete->text)
-			g_free (complete->text);
+		g_free (complete->text);
 		complete->text = g_strdup (text);
 	}
 
