@@ -60,7 +60,7 @@ typedef struct {
 } GnmSOFilled;
 typedef SheetObjectClass GnmSOFilledClass;
 
-#ifdef WITH_GTK
+#ifdef GNM_WITH_GTK
 #include <goffice/cut-n-paste/foocanvas/foo-canvas.h>
 static void
 so_filled_view_destroy (SheetObjectView *sov)
@@ -124,7 +124,7 @@ static GSF_CLASS_FULL (FilledFooView, so_filled_foo_view,
 	NULL, NULL, NULL, NULL,
 	NULL, FOO_TYPE_CANVAS_GROUP, 0,
 	GSF_INTERFACE (so_filled_foo_view_init, SHEET_OBJECT_VIEW_TYPE))
-#endif /* WITH_GTK */
+#endif /* GNM_WITH_GTK */
 
 /*****************************************************************************/
 
@@ -152,7 +152,7 @@ sof_default_style (void)
 }
 
 /*****************************************************************************/
-#ifdef WITH_GTK
+#ifdef GNM_WITH_GTK
 #include <sheet-control-gui.h>
 #include <dialogs/dialogs.h>
 #include <gnumeric-simple-canvas.h>
@@ -244,46 +244,45 @@ gnm_so_filled_new_view (SheetObject *so, SheetObjectViewContainer *container)
 	return gnm_pane_object_register (so, FOO_CANVAS_ITEM (group), TRUE);
 }
 
-#endif /* WITH_GTK */
+#endif /* GNM_WITH_GTK */
 
 static void
-gnm_so_filled_draw_cairo (SheetObject const *so, gpointer data,
+gnm_so_filled_draw_cairo (SheetObject const *so, cairo_t *cr,
 			  double width, double height)
 {
 	GnmSOFilled *sof = GNM_SO_FILLED (so);
-	cairo_t *cairo = (cairo_t*) data;
 	GogStyle const *style = sof->style;
 	cairo_pattern_t *pat = NULL;
 
-	cairo_new_path (cairo);
+	cairo_new_path (cr);
 	if (sof->is_oval) {
-		cairo_save (cairo);
-		cairo_scale (cairo, width, height);
-		cairo_arc (cairo, .5, .5, .5, 0., 2 * M_PI);
-		cairo_restore (cairo);
+		cairo_save (cr);
+		cairo_scale (cr, width, height);
+		cairo_arc (cr, .5, .5, .5, 0., 2 * M_PI);
+		cairo_restore (cr);
 	} else {
-		cairo_move_to (cairo, 0., 0.);
-		cairo_line_to (cairo, width, 0.);
-		cairo_line_to (cairo, width, height);
-		cairo_line_to (cairo, 0., height);
-		cairo_line_to (cairo, 0., 0.);
-		cairo_close_path (cairo);
+		cairo_move_to (cr, 0., 0.);
+		cairo_line_to (cr, width, 0.);
+		cairo_line_to (cr, width, height);
+		cairo_line_to (cr, 0., height);
+		cairo_line_to (cr, 0., 0.);
+		cairo_close_path (cr);
 	}
 	/* Fill the shape */
-	pat = gog_style_create_cairo_pattern (style, cairo);
+	pat = gog_style_create_cairo_pattern (style, cr);
 	if (pat) {
-		cairo_set_source (cairo, pat);
-		cairo_fill_preserve (cairo);
+		cairo_set_source (cr, pat);
+		cairo_fill_preserve (cr);
 		cairo_pattern_destroy (pat);
 	}
 	/* Draw the line */
-	cairo_set_line_width (cairo, (style->outline.width)? style->outline.width: 1.);
-	cairo_set_source_rgba (cairo,
+	cairo_set_line_width (cr, (style->outline.width)? style->outline.width: 1.);
+	cairo_set_source_rgba (cr,
 		UINT_RGBA_R(style->outline.color),
 		UINT_RGBA_B(style->outline.color),
 		UINT_RGBA_G(style->outline.color),
 		UINT_RGBA_A(style->outline.color));
-	cairo_stroke (cairo);
+	cairo_stroke (cr);
 }
 
 static gboolean
@@ -485,10 +484,10 @@ gnm_so_filled_class_init (GObjectClass *gobject_class)
 	so_class->rubber_band_directly	= TRUE;
 	so_class->xml_export_name	= "SheetObjectFilled";
 
-#ifdef WITH_GTK
+#ifdef GNM_WITH_GTK
 	so_class->new_view		= gnm_so_filled_new_view;
 	so_class->user_config		= gnm_so_filled_user_config;
-#endif /* WITH_GTK */
+#endif /* GNM_WITH_GTK */
 
 	so_class->draw_cairo	= gnm_so_filled_draw_cairo;
 
