@@ -79,7 +79,7 @@ typedef SheetObjectClass GnmSOLineClass;
 
 static SheetObjectClass *gnm_so_line_parent_class;
 
-#ifdef WITH_GTK
+#ifdef GNM_WITH_GTK
 #include <goffice/cut-n-paste/foocanvas/foo-canvas.h>
 #include <goffice/cut-n-paste/foocanvas/foo-canvas-line.h>
 #include <goffice/cut-n-paste/foocanvas/foo-canvas-util.h>
@@ -123,7 +123,7 @@ static GSF_CLASS_FULL (LineFooView, so_line_foo_view,
 	NULL, NULL, NULL, NULL,
 	NULL, FOO_TYPE_CANVAS_LINE, 0,
 	GSF_INTERFACE (so_line_foo_view_init, SHEET_OBJECT_VIEW_TYPE))
-#endif /* WITH_GTK */
+#endif /* GNM_WITH_GTK */
 enum {
 	SOL_PROP_0,
 	SOL_PROP_STYLE,
@@ -143,7 +143,7 @@ sol_default_style (void)
 	return res;
 }
 
-#ifdef WITH_GTK
+#ifdef GNM_WITH_GTK
 #include <sheet-control-gui.h>
 #include <dialogs/dialogs.h>
 #include <gnumeric-simple-canvas.h>
@@ -202,14 +202,13 @@ gnm_so_line_new_view (SheetObject *so, SheetObjectViewContainer *container)
 	return gnm_pane_object_register (so, item, TRUE);
 }
 
-#endif /* WITH_GTK */
+#endif /* GNM_WITH_GTK */
 
 static void
-gnm_so_line_draw_cairo (SheetObject const *so, gpointer data,
+gnm_so_line_draw_cairo (SheetObject const *so, cairo_t *cr,
 			double width, double height)
 {
 	GnmSOLine *sol = GNM_SO_LINE (so);
-	cairo_t *cairo = (cairo_t*) data;
 	GogStyleLine const *style = &sol->style->line;
 	double x1, y1, x2, y2;
 
@@ -248,8 +247,8 @@ gnm_so_line_draw_cairo (SheetObject const *so, gpointer data,
 		return;
 	}
 
-	cairo_set_line_width (cairo, (style->width)? style->width: 1.);
-	cairo_set_source_rgba (cairo,
+	cairo_set_line_width (cr, (style->width)? style->width: 1.);
+	cairo_set_source_rgba (cr,
 		UINT_RGBA_R(style->color),
 		UINT_RGBA_B(style->color),
 		UINT_RGBA_G(style->color),
@@ -260,18 +259,18 @@ gnm_so_line_draw_cairo (SheetObject const *so, gpointer data,
 
 		phi = atan2 (y2 - y1, x2 - x1) - M_PI_2;
 
-		cairo_save (cairo);
-		cairo_translate (cairo, x2, y2);
-		cairo_rotate (cairo, phi);
-		cairo_set_line_width (cairo, 1.0);
-		cairo_new_path (cairo);
-		cairo_move_to (cairo, 0.0, 0.0);
-		cairo_line_to (cairo, -sol->end_arrow.c, -sol->end_arrow.b);
-		cairo_line_to (cairo, 0.0, -sol->end_arrow.a);
-		cairo_line_to (cairo, sol->end_arrow.c, -sol->end_arrow.b);
-		cairo_close_path (cairo);
-		cairo_fill (cairo);
-		cairo_restore (cairo);
+		cairo_save (cr);
+		cairo_translate (cr, x2, y2);
+		cairo_rotate (cr, phi);
+		cairo_set_line_width (cr, 1.0);
+		cairo_new_path (cr);
+		cairo_move_to (cr, 0.0, 0.0);
+		cairo_line_to (cr, -sol->end_arrow.c, -sol->end_arrow.b);
+		cairo_line_to (cr, 0.0, -sol->end_arrow.a);
+		cairo_line_to (cr, sol->end_arrow.c, -sol->end_arrow.b);
+		cairo_close_path (cr);
+		cairo_fill (cr);
+		cairo_restore (cr);
 
 		/* Make the line shorter so that the arrow won't be
 		 * on top of a (perhaps quite fat) line.  */
@@ -279,11 +278,11 @@ gnm_so_line_draw_cairo (SheetObject const *so, gpointer data,
 		y2 -= sol->end_arrow.a * cos (phi);
 	}
 
-	cairo_set_line_width (cairo, style->width);
-	cairo_new_path (cairo);
-	cairo_move_to (cairo, x1, y1);
-	cairo_line_to (cairo, x2, y2);
-	cairo_stroke (cairo);
+	cairo_set_line_width (cr, style->width);
+	cairo_new_path (cr);
+	cairo_move_to (cr, x1, y1);
+	cairo_line_to (cr, x2, y2);
+	cairo_stroke (cr);
 }
 
 static gboolean
@@ -462,11 +461,11 @@ gnm_so_line_class_init (GObjectClass *gobject_class)
 	so_class->rubber_band_directly  = TRUE;
 	so_class->xml_export_name	= "SheetObjectGraphic";
 
-#ifdef WITH_GTK
+#ifdef GNM_WITH_GTK
 	so_class->draw_cairo	= gnm_so_line_draw_cairo;
 	so_class->user_config		= gnm_so_line_user_config;
 	so_class->new_view		= gnm_so_line_new_view;
-#endif /* WITH_GTK */
+#endif /* GNM_WITH_GTK */
 
         g_object_class_install_property (gobject_class, SOL_PROP_STYLE,
                  g_param_spec_object ("style", NULL, NULL, GOG_STYLE_TYPE,
