@@ -72,6 +72,7 @@ typedef struct {
 	PrintRange pr;
 	guint to, from;
 	guint last_pagination;
+	HFRenderInfo *hfi;
 } PrintingInstance;
 
 typedef struct {
@@ -89,6 +90,13 @@ typedef struct {
 	gint first_rep_rows;
 } SheetPageRange;
 
+typedef struct {
+	gint rc;
+	gint count;
+	gint first_rep;
+	gint n_rep;
+} PaginationInfo;
+
 
 gboolean gnm_print_debug = FALSE;
 
@@ -96,6 +104,7 @@ static PrintingInstance *
 printing_instance_new (void)
 {
 	PrintingInstance * pi = g_new0 (PrintingInstance,1);
+	pi->hfi = hf_render_info_new ();
 
 	return pi;
 }
@@ -105,6 +114,7 @@ printing_instance_delete (PrintingInstance *pi)
 {
 	go_list_free_custom (pi->gnmSheets, g_free);
 	go_list_free_custom (pi->gnmSheetRanges, g_free);
+	hf_render_info_destroy (pi->hfi);
 	g_free (pi);
 }
 
@@ -115,7 +125,7 @@ print_sheet_objects (GtkPrintContext   *context,
 		     GnmRange *range,
 		     double base_x, double base_y)
 {
-	GSList *ptr;
+	GSList *ptr, *objects;
 	double width, height;
 
 	g_return_if_fail (IS_SHEET (sheet));
@@ -140,7 +150,9 @@ print_sheet_objects (GtkPrintContext   *context,
 				 width, height);
 	cairo_clip (cr);
 
-	for (ptr = sheet->sheet_objects; ptr; ptr = ptr->next) {
+	objects = g_slist_reverse (g_slist_copy (sheet->sheet_objects));
+
+	for (ptr = objects; ptr; ptr = ptr->next) {
 		SheetObject *so = SHEET_OBJECT (ptr->data);
 		GnmRange const *r = &so->anchor.cell_bound;
 
@@ -175,6 +187,9 @@ print_sheet_objects (GtkPrintContext   *context,
 		sheet_object_draw_cairo (so, (gpointer)cr, sheet->text_is_rtl);
 		cairo_restore (cr);
 	}
+
+	g_slist_free (objects);
+
 	cairo_restore (cr);
 }
 
@@ -301,28 +316,23 @@ print_page_row_headers (GtkPrintContext   *context, PrintingInstance * pi,
 	pango_font_description_free (desc);
 }
 
-#if 0
-
 static PangoLayout *
-ensure_decoration_layout (PrintJobInfo *pj)
+ensure_decoration_layout (GtkPrintContext   *context)
 {
-	if (!pj->decoration_layout) {
-		PangoLayout *layout = gnome_print_pango_create_layout (pj->print_context);
-		/*
-		 * Copy the style so we don't leave a cached GnmFont in the
-		 * prefs object.
-		 */
-		GnmStyle *style = gnm_style_dup (gnm_app_prefs->printer_decoration_font);
-		GnmFont *font = gnm_style_get_font
-			(style,
-			 pango_layout_get_context (layout),
-			 1.);
+	GnmStyle *style;
+	GnmFont *font;
+	PangoLayout *layout;
 
-		pj->decoration_layout = layout;
-		pango_layout_set_font_description (layout, font->go.font->desc);
-		gnm_style_unref (style);
-	}
-	return pj->decoration_layout;
+	layout = gtk_print_context_create_pango_layout (context);
+	style = gnm_style_dup (gnm_app_prefs->printer_decoration_font);
+	font = gnm_style_get_font
+		(style,
+		 pango_layout_get_context (layout),
+		 1.);
+	pango_layout_set_font_description (layout, font->go.font->desc);
+	gnm_style_unref (style);
+
+	return layout;
 }
 
 
@@ -339,156 +349,62 @@ ensure_decoration_layout (PrintJobInfo *pj)
  * the rectangle.
  */
 static void
-print_hf_element (PrintJobInfo const *pj, Sheet const *sheet, char const *format,
-		  PangoAlignment side, double y, gboolean align_bottom)
+print_hf_element (GtkPrintContext   *context, cairo_t *cr, Sheet const *sheet, 
+		  char const *format,
+		  PangoAlignment side, gdouble width, gboolean align_bottom, 
+		  HFRenderInfo *hfi)
 {
-	PrintInformation const *pi = sheet->print_info;
+	PangoLayout *layout;
+	
+	gdouble text_height = 0.;
 	char *text;
 
-	g_return_if_fail (pj != NULL);
-	g_return_if_fail (pj->render_info != NULL);
-
 	if (format == NULL)
 		return;
-	text = hf_format_render (format, pj->render_info, HF_RENDER_PRINT);
 
-	if (text && text[0]) {
-		double header = 0, footer = 0, left = 0, right = 0;
-		PangoLayout *layout = ensure_decoration_layout ((PrintJobInfo *)pj);
+	text = hf_format_render (format, hfi, HF_RENDER_PRINT);
+	
+	if (text == NULL)
+		return;
 
-		print_info_get_margins (pi, &header, &footer, &left, &right);
-		pango_layout_set_alignment (layout, side);
-		pango_layout_set_width (layout, (pj->width - left - right) * PANGO_SCALE);
-		pango_layout_set_text (layout, text, -1);
-
-		if (align_bottom) {
-			int height;
-			pango_layout_get_size (layout, NULL, &height);
-			y += height / (double)PANGO_SCALE;
-		}
-
-		gnome_print_moveto (pj->print_context, left, y);
-		gnome_print_pango_layout (pj->print_context, layout);
+	layout = ensure_decoration_layout (context);
+	
+	pango_layout_set_text (layout, text, -1);
+	pango_layout_set_width (layout, width * PANGO_SCALE);
+	pango_layout_set_alignment (layout, side);
+	
+	if (align_bottom) {
+		gint layout_height = 0;
+		pango_layout_get_size (layout, NULL, &layout_height);
+		text_height = (gdouble)layout_height / PANGO_SCALE;
 	}
-	g_free (text);
+		
+	cairo_move_to (cr, 0., - text_height);
+	pango_cairo_show_layout (cr, layout);
+		
+	g_object_unref (layout);
+	g_free(text);
 }
 
 /*
  * print_hf_line
- * @pj:     printing context
  * @hf:     header/footer descriptor
- * @y:      vertical position
- * @left:   left coordinate of clip rectangle
- * @bottom: bottom coordinate of clip rectangle
- * @right:  right coordinate of clip rectangel
- * @top:    top coordinate of clip rectangle
+ * @align_bottom:      vertical position (whether to print above or below
+ * @width:  width of header line
  *
  * Print a header/footer line.
  *
- * Position at y, and clip to rectangle. If gnm_print_debug is TRUE, display
- * the rectangle.
  */
 static void
-print_hf_line (PrintJobInfo const *pj, Sheet const *sheet,
-	       PrintHF const *hf,
-	       double y,
-	       double left, double bottom, double right, double top,
-	       gboolean align_bottom)
+print_hf_line (GtkPrintContext   *context, cairo_t *cr, Sheet const *sheet, 
+	       PrintHF const *hf, gboolean align_bottom, gdouble width, HFRenderInfo *hfi)
 {
-	/* Check if there's room to print. top and bottom are on the clip
-	 * path, so we are actually requiring room for a 6x4 pt
-	 * character. */
-	if (ABS (top - bottom) < 8)
-		return;
-	if (ABS (left - right) < 6)
-		return;
-
-	gnome_print_gsave (pj->print_context);
-
-	gnome_print_setrgbcolor (pj->print_context, 0, 0, 0);
-
-	gnm_print_make_rect_path (pj->print_context,
-				   left, bottom, right, top);
-
-#ifndef NO_DEBUG_PRINT
-	if (gnm_print_debug) {
-		static const double dash[] = {1.0, 2.0, 3.0, 4.0, 5.0, 6.0};
-		static gint n_dash = G_N_ELEMENTS (dash);
-
-		gnome_print_gsave (pj->print_context);
-		gnome_print_setdash (pj->print_context, n_dash, dash, 0.0);
-		gnome_print_stroke  (pj->print_context);
-		gnome_print_grestore (pj->print_context);
-	}
-#endif
-	/* Clip the header or footer */
-	gnome_print_clip      (pj->print_context);
-
-	print_hf_element (pj, sheet, hf->left_format, PANGO_ALIGN_LEFT, y, align_bottom);
-	print_hf_element (pj, sheet, hf->middle_format, PANGO_ALIGN_CENTER, y, align_bottom);
-	print_hf_element (pj, sheet, hf->right_format, PANGO_ALIGN_RIGHT, y, align_bottom);
-
-	gnome_print_grestore (pj->print_context);
+	print_hf_element (context, cr, sheet, hf->left_format, PANGO_ALIGN_LEFT, width, align_bottom, hfi);
+	print_hf_element (context, cr, sheet, hf->middle_format, PANGO_ALIGN_CENTER, width, align_bottom, hfi);
+	print_hf_element (context, cr, sheet, hf->right_format, PANGO_ALIGN_RIGHT, width, align_bottom, hfi);
 }
 
-/*
- * print_headers
- * @pj: printing context
- * Print headers
- *
- * Align ascenders flush with inside of top margin.
- */
-static void
-print_headers (PrintJobInfo const *pj, Sheet const *sheet)
-{
-	PrintInformation const *pi = sheet->print_info;
-	PrintMargins const *pm = &pi->margin;
-	double top, bottom, y;
-	double header = 0, footer = 0, left = 0, right = 0;
 
-	print_info_get_margins (pi, &header, &footer, &left, &right);
-
-	y = pj->height - header;
-	top    =  1 + pj->height - MIN (header, pm->top.points);
-	bottom = -1 + pj->height - MAX (header, pm->top.points);
-
-	print_hf_line (pj, sheet, pi->header, y,
-		       -1 + left, bottom,
-		       pj->width - right, top,
-		       FALSE);
-}
-
-/*
- * print_footers
- * @pj: printing context
- * Print footers
- *
- * Align descenders flush with inside of bottom margin.
- */
-static void
-print_footers (PrintJobInfo const *pj, Sheet const *sheet)
-{
-	PrintInformation const *pi = sheet->print_info;
-	PrintMargins const *pm = &pi->margin;
-	double top, bottom, y;
-	double header = 0, footer = 0, left = 0, right = 0;
-
-	print_info_get_margins (pi, &header, &footer, &left, &right);
-
-	y = footer;
-	top    =  1 + MAX (footer, pm->bottom.points);
-	bottom = -1 + MIN (footer, pm->bottom.points);
-
-	/* Clip path for the header
-	 * NOTE : postscript clip paths exclude the border, gdk includes it.
-	 */
-	print_hf_line (pj, sheet, pi->footer, y,
-		       -1 + left, bottom,
-		       pj->width - right, top,
-		       TRUE);
-}
-
-#endif
 
 /**
  * print_page:
@@ -562,15 +478,27 @@ print_page (GtkPrintOperation *operation,
 			        gsr->first_rep_rows + gsr->n_rep_rows - 1);
        }
 
-	
 	print_height = main_height + col_header_height + rep_row_height;
 	print_width = main_width + row_header_width + rep_col_width;
 
-/* printing header  */
-
-
-/* printing footer  */
-
+	/* printing header  */
+	
+	if (edge_to_below_header > header) {
+		cairo_save (cr);
+		print_hf_line (context, cr, sheet, pinfo->header, 
+			       FALSE, width, pi->hfi);
+		cairo_restore (cr);
+	}
+	
+	/* printing footer  */
+	
+	if (edge_to_above_footer > footer) {
+		cairo_save (cr);
+		cairo_translate (cr, 0, height + header + footer);
+		print_hf_line (context, cr, sheet, pinfo->footer, TRUE, width, 
+			       pi->hfi);
+		cairo_restore (cr);
+	}
 
 /* setting up content area */
 	cairo_save (cr);
@@ -685,185 +613,6 @@ compute_group (Sheet const *sheet,
 	return count;
 }
 
-static int
-compute_n_pages (Sheet const *sheet,
-	       int start, int end, double usable,
-	       ColRowInfo const *(get_info)(Sheet const *sheet, int const p))
-{
-	int count = 0;
-
-	while (start <= end) {
-		start += compute_group (sheet, start, end, usable, get_info);
-		count++;
-	}
-
-	return count;
-}
-
-/* computer_scale_fit_to
- * Computes the scaling needed to fit all the rows or columns into the @usable
- * area.
- * This function is called when printing, and the user has selected the 'fit-to'
- * printing option. It will adjust the internal x and y scaling values to
- * make the sheet fit the desired number of pages, as suggested by the user.
- * It will only reduce the scaling to fit inside a page, not enlarge.
- */
-static double
-compute_scale_fit_to (Sheet const *sheet,
-		      int start, int end, double usable,
-		      ColRowInfo const *(get_info)(Sheet const *sheet, int const p),
-		      gint pages, double max_percent, double extent, double header)
-{
-	double max_p, min_p;
-	gint   max_pages, min_pages;
-
-	/* This means to take whatever space is needed.  */
-	if (pages <= 0)
-		return max_percent;
-
-	/* We can handle a single page easily: */
-	if (pages == 1) {
-		max_p = usable/(header + extent + 2.);
-		return ((max_p > max_percent) ? max_percent : max_p);
-	}
-
-	/* There is no easy way to calculate really which percentage is needed */
-	/* without in fact allocating the cols/rows to pages.                  */
-
-	/* We first calculate the max percentage needed */
-
-	max_p = (pages * usable)/(extent + pages * header);
-	if (max_p > max_percent)
-		max_p = max_percent;
-
-	max_pages = compute_n_pages (sheet, start, end, usable/max_p - header, get_info);
-
-	if (max_pages == pages)
-		return max_p;
-
-	/* The we calculate the min percentage */
-
-	min_p = usable/(extent + header);
-	if (min_p > max_percent)
-		min_p = max_percent;
-
-	min_pages = compute_n_pages (sheet, start, end, usable/min_p - header, get_info);
-
-
-	/* And then we pick the middle until the percentage is within 0.1% of */
-	/* the desired percentage */
-
-	while (max_p - min_p > 0.001) {
-		double cur_p = (max_p + min_p) / 2.;
-		int cur_pages = compute_n_pages (sheet, start, end, usable/cur_p - header, get_info);
-
-		if (cur_pages > pages) {
-			max_pages = cur_pages;
-			max_p = cur_p;
-		} else {
-			min_pages = cur_pages;
-			min_p = cur_p;
-		}
-	}
-
-	return min_p;
-}
-
-#define COL_FIT(col) (col >= SHEET_MAX_COLS ? (SHEET_MAX_COLS-1) : col)
-#define ROW_FIT(row) (row >= SHEET_MAX_ROWS ? (SHEET_MAX_ROWS-1) : row)
-
-#if 0
-
-static double
-print_range_used_units (Sheet const *sheet, gboolean compute_rows,
-			PrintRepeatRange const *range)
-{
-	GnmRange const *r = &range->range;
-	if (compute_rows)
-		return sheet_row_get_distance_pts
-			(sheet, r->start.row, r->end.row+1);
-	else
-		return sheet_col_get_distance_pts
-			(sheet, r->start.col, r->end.col+1);
-}
-
-static void
-print_job_info_init_sheet (PrintJobInfo *pj, Sheet const *sheet)
-{
-	PrintInformation const *pi = sheet->print_info;
-	PrintMargins const *pm = &sheet->print_info->margin;
-	double header = 0, footer = 0, left = 0, right = 0;
-
-	if (!gnome_print_config_get_page_size (pj->gp_config, &pj->width, &pj->height))
-		pj->width = pj->height = 1.;
-
-	print_info_get_margins (pi, &header, &footer, &left, &right);
-	pj->x_points = pj->width - (left + right);
-	pj->y_points = pj->height -
-		(MAX (pm->top.points, header) +
-		 MAX (pm->bottom.points, footer));
-
-	if (pi->print_titles) {
-		pj->titles_used_x = sheet->cols.default_style.size_pts;
-		pj->titles_used_y = sheet->rows.default_style.size_pts;
-	} else {
-		pj->titles_used_x = 0;
-		pj->titles_used_y = 0;
-	}
-
-	pj->repeat_rows_used_y = (pi->repeat_top.use)
-	    ? print_range_used_units (sheet, TRUE, &pi->repeat_top)
-	    : 0.;
-	pj->repeat_cols_used_x = (pi->repeat_left.use)
-	    ? print_range_used_units (sheet, FALSE, &pi->repeat_left)
-	    : 0.;
-
-	pj->render_info->sheet = sheet;
-}
-
-/*
- * code to count the number of pages that will be printed.
- * Unfortuantely a lot of data here is calculated again when you
- * actually print the page ...
- */
-
-typedef struct _PageCountInfo {
-	int pages;
-	PrintJobInfo *pj;
-	GnmRange r;
-	int current_output_sheet;
-} PageCountInfo;
-
-#endif
-
-static void
-compute_sheet_pages_add_sheet (PrintingInstance * pi, Sheet const *sheet, gboolean selection,
-                     gboolean ignore_printarea)
-{
-	SheetPrintInfo *spi = g_new (SheetPrintInfo, 1);
-
-	spi->sheet = (Sheet *) sheet;
-	spi->selection = selection;
-	spi->ignore_printarea = ignore_printarea;
-	pi->gnmSheets = g_list_append(pi->gnmSheets, spi);
-}
-
-static void
-compute_sheet_pages_add_range (PrintingInstance * pi, Sheet const *sheet,
-			       GnmRange *r, gint n_rep_cols, gint n_rep_rows,
-			       gint first_rep_rows, gint first_rep_cols)
-{
-		SheetPageRange *gsr = g_new (SheetPageRange,1);
-
-		gsr->n_rep_cols = n_rep_cols;
-		gsr->n_rep_rows = n_rep_rows;
-		gsr->first_rep_rows = first_rep_rows;
-		gsr->first_rep_cols = first_rep_cols;
-		gsr->range = *r;
-		gsr->sheet = (Sheet *) sheet;
-		pi->gnmSheetRanges = g_list_append(pi->gnmSheetRanges, gsr);
-}
-
 static void
 adjust_repetition (Sheet const *sheet,
 		   gint i,
@@ -888,145 +637,246 @@ adjust_repetition (Sheet const *sheet,
 	}
 }
 
-static gboolean
+static gint 
+paginate (GSList **paginationInfo,
+	  Sheet const *sheet,
+	  gint start, gint end,
+	  gdouble usable, gboolean repeat, gint repeat_start, gint repeat_end,
+	  double (sheet_get_distance_pts) (Sheet const *sheet, int from, int to),
+	  ColRowInfo const *(get_info)(Sheet const *sheet, int const p))
+{
+	GSList *list = NULL;
+	int rc = start;
+	gint n_rep = 0, first_rep = 0;
+	gdouble repeating = 0.;
+	gint page_count = 0;
+
+	if (repeat) {
+		first_rep = repeat_start;
+		n_rep = repeat_end - first_rep + 1;
+		repeating = sheet_get_distance_pts (sheet, first_rep, first_rep + n_rep);
+	} 
+
+	while (rc <= end) {
+		int count;
+		PaginationInfo *item;
+ 
+		gdouble repeating_used = 0.;
+		gint n_rep_used = 0, first_rep_used = 0;
+
+		adjust_repetition (sheet, rc,
+				   first_rep, n_rep,
+				   repeating,
+				   &first_rep_used, &n_rep_used,
+				   &repeating_used,
+				   sheet_get_distance_pts);
+
+		count = compute_group (sheet, rc, end,
+				       usable - repeating_used, 
+				       get_info);
+	
+		if (paginationInfo) {
+			item = g_new (PaginationInfo,1);
+			item->rc = rc;
+			item->count = count;
+			item->first_rep = first_rep_used;
+			item->n_rep = n_rep_used;
+
+			list = g_slist_prepend (list, item);
+		}
+		page_count++;
+
+		rc += count;
+	}
+
+	if (paginationInfo) {
+		list = g_slist_reverse (list);
+
+		*paginationInfo = list;
+	}
+
+	return page_count;
+}
+
+/* computer_scale_fit_to
+ * Computes the scaling needed to fit all the rows or columns into the @usable
+ * area.
+ * This function is called when printing, and the user has selected the 'fit-to'
+ * printing option. It will adjust the internal x and y scaling values to
+ * make the sheet fit the desired number of pages, as suggested by the user.
+ * It will only reduce the scaling to fit inside a page, not enlarge.
+ */
+static double
+compute_scale_fit_to (Sheet const *sheet,
+		      int start, int end, double usable,
+		      ColRowInfo const *(get_info)(Sheet const *sheet, int const p),
+		      double (get_distance_pts) (Sheet const *sheet, int from, int to),
+		      gint pages, double max_percent, double header,
+		      gboolean repeat, gint repeat_start, gint repeat_end)
+{
+	double max_p, min_p;
+	gint   max_pages, min_pages;
+	double extent;
+
+	extent = get_distance_pts (sheet, start, end + 1);
+
+	/* If the repeating columns are not included we should add them */
+	if (repeat && (repeat_start < start))
+		extent += get_distance_pts (sheet, repeat_start, 
+					    (repeat_end < start) ? (repeat_end + 1) : start);
+
+	/* This means to take whatever space is needed.  */
+	if (pages <= 0)
+		return max_percent;
+
+	/* We can handle a single page easily: */
+	if (pages == 1) {
+		max_p = usable/(header + extent + 2.);
+		return ((max_p > max_percent) ? max_percent : max_p);
+	}
+
+	/* There is no easy way to calculate really which percentage is needed */
+	/* without in fact allocating the cols/rows to pages.                  */
+
+	/* We first calculate the max percentage needed */
+
+	max_p = (pages * usable)/(extent + pages * header);
+	if (max_p > max_percent)
+		max_p = max_percent;
+
+	max_pages = paginate (NULL, sheet, start, end, usable/max_p - header,
+			      repeat, repeat_start, repeat_end,
+			      get_distance_pts, get_info);
+
+	if (max_pages == pages)
+		return max_p;
+
+	/* The we calculate the min percentage */
+
+	min_p = usable/(extent + header);
+	if (min_p > max_percent)
+		min_p = max_percent;
+
+	min_pages = paginate (NULL, sheet, start, end, usable/min_p - header,
+			      repeat, repeat_start, repeat_end,
+			      get_distance_pts, get_info);
+
+
+	/* And then we pick the middle until the percentage is within 0.1% of */
+	/* the desired percentage */
+
+	while (max_p - min_p > 0.001) {
+		double cur_p = (max_p + min_p) / 2.;
+		int cur_pages = paginate (NULL, sheet, start, end, usable/cur_p - header,
+					  repeat, repeat_start, repeat_end,
+					  get_distance_pts, get_info);
+
+		if (cur_pages > pages) {
+			max_pages = cur_pages;
+			max_p = cur_p;
+		} else {
+			min_pages = cur_pages;
+			min_p = cur_p;
+		}
+	}
+
+	return min_p;
+}
+
+#define COL_FIT(col) (col >= SHEET_MAX_COLS ? (SHEET_MAX_COLS-1) : col)
+#define ROW_FIT(row) (row >= SHEET_MAX_ROWS ? (SHEET_MAX_ROWS-1) : row)
+
+static void
+compute_sheet_pages_add_sheet (PrintingInstance * pi, Sheet const *sheet, gboolean selection,
+                     gboolean ignore_printarea)
+{
+	SheetPrintInfo *spi = g_new (SheetPrintInfo, 1);
+
+	spi->sheet = (Sheet *) sheet;
+	spi->selection = selection;
+	spi->ignore_printarea = ignore_printarea;
+	pi->gnmSheets = g_list_append(pi->gnmSheets, spi);
+}
+
+static void
+compute_sheet_pages_add_range (PrintingInstance * pi, Sheet const *sheet,
+			       GnmRange *r, gint n_rep_cols, gint n_rep_rows,
+			       gint first_rep_cols, gint first_rep_rows)
+{
+		SheetPageRange *gsr = g_new (SheetPageRange,1);
+
+		gsr->n_rep_cols = n_rep_cols;
+		gsr->n_rep_rows = n_rep_rows;
+		gsr->first_rep_rows = first_rep_rows;
+		gsr->first_rep_cols = first_rep_cols;
+		gsr->range = *r;
+		gsr->sheet = (Sheet *) sheet;
+		pi->gnmSheetRanges = g_list_append(pi->gnmSheetRanges, gsr);
+}
+
+
+
+static void
 compute_sheet_pages_down_then_across (PrintingInstance * pi,
 				      Sheet const *sheet,
-				      GnmRange const *r,
-				      gdouble usable_x,
-				      gdouble usable_y,
-				      gdouble col_header_height,
-				      gdouble row_header_width,
-				      gint n_rep_cols,
-				      gint n_rep_rows,
-				      gint first_rep_rows,
-				      gint first_rep_cols,
-				      gdouble repeating_x, 
-				      gdouble repeating_y)
+				      GSList *column_pagination,
+				      GSList *row_pagination)
 {
-	int col = r->start.col;
+	GnmRange range;
+	GSList *c_list = column_pagination;
 
-	while (col <= r->end.col) {
-		int row = r->start.row;
-		int col_count;
-		gdouble repeating_x_used = 0.;
-		gint n_rep_cols_used = 0, first_rep_cols_used = 0;
+	while (c_list) {
+		PaginationInfo *c_info = c_list->data;
+		GSList *r_list = row_pagination;
 
-		adjust_repetition (sheet, col,
-				   first_rep_cols, n_rep_cols,
-				   repeating_x,
-				   &first_rep_cols_used, &n_rep_cols_used,
-				   &repeating_x_used,
-				   sheet_col_get_distance_pts);
-
-		col_count = compute_group (sheet, col, r->end.col,
-					   usable_x - row_header_width - repeating_x_used, 
-					   sheet_col_get_info);
-
-		while (row <= r->end.row) {
-			int row_count;
-			gdouble repeating_y_used = 0.;
-			gint n_rep_rows_used = 0, first_rep_rows_used = 0;
-
-			adjust_repetition (sheet, row,
-					   first_rep_rows, n_rep_rows,
-					   repeating_y,
-					   &first_rep_rows_used, &n_rep_rows_used,
-					   &repeating_y_used,
-					   sheet_row_get_distance_pts);
-
-			row_count = compute_group (sheet, row, r->end.row,
-						   usable_y - col_header_height - repeating_y_used, 
-						   sheet_row_get_info);
-
-			{
-				GnmRange range;
-				range_init (&range, COL_FIT (col), ROW_FIT (row),
-					    COL_FIT (col + col_count - 1),
-					    ROW_FIT (row + row_count - 1));
-				compute_sheet_pages_add_range (pi, sheet, &range, 
-							       n_rep_cols_used, n_rep_rows_used,
-							       first_rep_rows_used, first_rep_cols_used);
-			}
+		while (r_list) {
+			PaginationInfo *r_info = r_list->data;
 			
-			row += row_count;
+			range_init (&range, COL_FIT (c_info->rc), ROW_FIT (r_info->rc),
+				    COL_FIT (c_info->rc + c_info->count - 1),
+				    ROW_FIT (r_info->rc + r_info->count - 1));
+			compute_sheet_pages_add_range (pi, sheet, &range, 
+						       c_info->n_rep, r_info->n_rep,
+						       c_info->first_rep, r_info->first_rep);
+			r_list = r_list->next;
 		}
-		col += col_count;
+		c_list = c_list->next;
 	}
-
-	return FALSE;
 }
 
-static gboolean
+static void
 compute_sheet_pages_across_then_down (PrintingInstance * pi,
 				      Sheet const *sheet,
-				      GnmRange const *r,
-				      gdouble usable_x,
-				      gdouble usable_y,
-				      gdouble col_header_height,
-				      gdouble row_header_width,
-				      gint n_rep_cols,
-				      gint n_rep_rows,
-				      gint first_rep_rows,
-				      gint first_rep_cols,
-				      gdouble repeating_x, 
-				      gdouble repeating_y)
+				      GSList *column_pagination,
+				      GSList *row_pagination)
 {
-	int row = r->start.row;
+	GnmRange range;
+	GSList *r_list = row_pagination;
 
-	while (row <= r->end.row) {
-		int col = r->start.col;
-		int row_count;
-		gdouble repeating_y_used = 0.;
-		gint n_rep_rows_used = 0, first_rep_rows_used = 0;
+	while (r_list) {
+		PaginationInfo *r_info = r_list->data;
+		GSList *c_list = column_pagination;
 
-		adjust_repetition (sheet, row,
-				   first_rep_rows, n_rep_rows,
-				   repeating_y,
-				   &first_rep_rows_used, &n_rep_rows_used,
-				   &repeating_y_used,
-				   sheet_row_get_distance_pts);
-
-		row_count = compute_group (sheet, row, r->end.row,
-					   usable_y - col_header_height - repeating_y_used, 
-					   sheet_row_get_info);
-
-		while (col <= r->end.col) {
-			int col_count;
-			gdouble repeating_x_used = 0.;
-			gint n_rep_cols_used = 0, first_rep_cols_used = 0;
-
-			adjust_repetition (sheet, col,
-					   first_rep_cols, n_rep_cols,
-					   repeating_x,
-					   &first_rep_cols_used, &n_rep_cols_used,
-					   &repeating_x_used,
-					   sheet_col_get_distance_pts);
-
-			col_count = compute_group (sheet, col, r->end.col,
-						   usable_x - row_header_width - repeating_x_used, 
-						   sheet_col_get_info);
-
-			{
-				GnmRange range;
-				range_init (&range, COL_FIT (col), ROW_FIT (row),
-					    COL_FIT (col + col_count - 1),
-					    ROW_FIT (row + row_count - 1));
-				compute_sheet_pages_add_range (pi, sheet, &range, 
-							       n_rep_cols_used, n_rep_rows_used,
-							       first_rep_rows_used, first_rep_cols_used);
-			}
+		while (c_list) {
+			PaginationInfo *c_info = c_list->data;
 			
-			col += col_count;
+			range_init (&range, COL_FIT (c_info->rc), ROW_FIT (r_info->rc),
+				    COL_FIT (c_info->rc + c_info->count - 1),
+				    ROW_FIT (r_info->rc + r_info->count - 1));
+			compute_sheet_pages_add_range (pi, sheet, &range, 
+						       c_info->n_rep, r_info->n_rep,
+						       c_info->first_rep, r_info->first_rep);
+			c_list = c_list->next;
 		}
-		row += row_count;
+		r_list = r_list->next;
 	}
-
-	return FALSE;
 }
 
+
 /*
- * Returns TRUE if case of problems.
- */
+  return TRUE in case of trouble
+*/
+
 static gboolean
 compute_sheet_pages (GtkPrintContext   *context,
 		     PrintingInstance * pi,
@@ -1041,11 +891,11 @@ compute_sheet_pages (GtkPrintContext   *context,
 	gdouble row_header_width = 0.;
 	gdouble page_width, page_height;
 	gdouble top_margin, bottom_margin, edge_to_below_header, edge_to_above_footer;
-	gint n_rep_cols = 0, n_rep_rows = 0, first_rep_rows = 0, first_rep_cols = 0;
-	gint last_rep_cols = 0, last_rep_rows = 0;
 	gdouble px, py;
 	gdouble usable_x, usable_y;
-	gdouble repeating_x = 0., repeating_y = 0.;
+
+	GSList *column_pagination = NULL;
+	GSList *row_pagination = NULL;
 
 	if (pinfo->print_titles) {
 		col_header_height = sheet->rows.default_style.size_pts;
@@ -1077,22 +927,29 @@ compute_sheet_pages (GtkPrintContext   *context,
 	page_height -= ((edge_to_below_header - top_margin)
 			+ (edge_to_above_footer - bottom_margin));
 
-	if (pinfo->repeat_top.use) {
-		first_rep_rows = pinfo->repeat_top.range.start.row;
-		last_rep_rows = pinfo->repeat_top.range.end.row;
-		n_rep_rows = last_rep_rows - first_rep_rows + 1;
-		repeating_y = sheet_row_get_distance_pts 
-			(sheet, first_rep_rows,
-			 first_rep_rows + n_rep_rows);
-	} 
-	if (pinfo->repeat_left.use) {
-		first_rep_cols = pinfo->repeat_left.range.start.col;
-		last_rep_cols = pinfo->repeat_left.range.end.col;
-		n_rep_cols = last_rep_cols - first_rep_cols + 1;
-		repeating_x = sheet_col_get_distance_pts 
-			(sheet, first_rep_cols,
-			 first_rep_cols + n_rep_cols);
-	} 
+	if (pinfo->scaling.type == PRINT_SCALE_FIT_PAGES) {
+		/* Note that the resulting scale is independent from */
+		/* whether we print first down or across!            */
+		gdouble pxy;
+
+		pxy = compute_scale_fit_to (sheet, r.start.row, r.end.row,
+					    page_height, sheet_row_get_info,
+					    sheet_row_get_distance_pts,
+					    pinfo->scaling.dim.rows, 1.,
+					    col_header_height,
+					    pinfo->repeat_top.use, pinfo->repeat_top.range.start.row, 
+					    pinfo->repeat_top.range.end.row);
+		pxy = compute_scale_fit_to (sheet, r.start.col, r.end.col,
+					    page_width, sheet_col_get_info,
+					    sheet_col_get_distance_pts,
+					    pinfo->scaling.dim.cols, pxy,
+					    row_header_width,
+					    pinfo->repeat_left.use, pinfo->repeat_left.range.start.col, 
+					    pinfo->repeat_left.range.end.col);
+
+		pinfo->scaling.percentage.x = pxy * 100.;
+		pinfo->scaling.percentage.y = pxy * 100.;
+	}
 
 	px = pinfo->scaling.percentage.x / 100.;
 	py = pinfo->scaling.percentage.y / 100.;
@@ -1105,47 +962,28 @@ compute_sheet_pages (GtkPrintContext   *context,
 	usable_x   = page_width / px;
 	usable_y   = page_height / py;
 
-
-	if (pinfo->scaling.type == PRINT_SCALE_FIT_PAGES) {
-		/* Note that the resulting scale is independent from */
-		/* whether we print first down or across!            */
-		gdouble pxy;
-
-		pxy = compute_scale_fit_to (sheet, r.start.row, r.end.row,
-					    page_height, sheet_row_get_info,
-					    pinfo->scaling.dim.rows, 1.,
-					    sheet_row_get_distance_pts (sheet,
-									r.start.row,
-									r.end.row+1),
-					    col_header_height);
-		pxy = compute_scale_fit_to (sheet, r.start.col, r.end.col,
-					    page_width, sheet_col_get_info,
-					    pinfo->scaling.dim.cols, pxy,
-					    sheet_col_get_distance_pts (sheet,
-									r.start.col,
-									r.end.col+1),
-					    row_header_width);
-
-		pinfo->scaling.percentage.x = pxy * 100.;
-		pinfo->scaling.percentage.y = pxy * 100.;
-	}
-
+	paginate (&column_pagination, sheet, r.start.col, r.end.col, 
+		  usable_x - row_header_width,
+		  pinfo->repeat_left.use, pinfo->repeat_left.range.start.col, 
+		  pinfo->repeat_left.range.end.col, 
+		  sheet_col_get_distance_pts, sheet_col_get_info);
+	paginate (&row_pagination, sheet, r.start.row, r.end.row, 
+		  usable_y - col_header_height,
+		  pinfo->repeat_top.use, pinfo->repeat_top.range.start.row, 
+		  pinfo->repeat_top.range.end.row,
+		  sheet_row_get_distance_pts, sheet_row_get_info);
 
 	if (sheet->print_info->print_across_then_down)
-		return compute_sheet_pages_across_then_down
-			(pi, sheet, &r, 
-			 usable_x, usable_y,
-			 col_header_height, row_header_width,
-			 n_rep_cols, n_rep_rows,
-			 first_rep_rows, first_rep_cols,
-			 repeating_x, repeating_y);
-	else return compute_sheet_pages_down_then_across
-		     (pi, sheet, &r, 
-		      usable_x, usable_y,
-		      col_header_height, row_header_width,
-		      n_rep_cols, n_rep_rows,
-		      first_rep_rows, first_rep_cols,
-		      repeating_x, repeating_y);
+		compute_sheet_pages_across_then_down (pi, sheet, 
+						      column_pagination,row_pagination);
+	else
+		compute_sheet_pages_down_then_across (pi, sheet,
+						      column_pagination,row_pagination);
+
+	go_slist_free_custom (column_pagination, g_free);
+	go_slist_free_custom (row_pagination, g_free);
+	
+	return FALSE;
 }
 
 /*
@@ -1159,42 +997,44 @@ compute_pages (GtkPrintOperation *operation,
 	       guint from,
 	       guint to)
 {
-	Workbook * wb = pi->wb;
+	Workbook *wb = pi->wb;
 	guint i;
 	guint n;
 
 	switch (pr) {
 	case PRINT_ACTIVE_SHEET:
-		compute_sheet_pages_add_sheet (pi,
-			   pi->sheet, FALSE, FALSE);
+		compute_sheet_pages_add_sheet (pi, pi->sheet, FALSE, FALSE);
 		break;
 	case PRINT_ALL_SHEETS:
-		n =  workbook_sheet_count (wb);
-		for (i = 0; i < n; i++)
-			compute_sheet_pages_add_sheet (pi,
-					     workbook_sheet_by_index (wb, i),
-					     FALSE, FALSE);
+		n = workbook_sheet_count (wb);
+		for (i = 0; i < n; i++) {
+			Sheet *sheet = workbook_sheet_by_index (wb, i);
+			if (sheet->print_info->do_not_print)
+				continue;
+			compute_sheet_pages_add_sheet (pi, sheet,
+						       FALSE, FALSE);
+		}
 		break;
 	case PRINT_SHEET_RANGE:
-		n =  workbook_sheet_count (wb);
+		n = workbook_sheet_count (wb);
 		if (to > n)
 			to = n;
-		for (i = from - 1; i < to; i++)
-			compute_sheet_pages_add_sheet (pi,
-					     workbook_sheet_by_index (wb, i),
-					     FALSE, FALSE);
+		for (i = from - 1; i < to; i++){
+			Sheet *sheet = workbook_sheet_by_index (wb, i);
+			if (sheet->print_info->do_not_print)
+				continue;
+			compute_sheet_pages_add_sheet (pi, sheet,
+						       FALSE, FALSE);
+		}
 		break;
 	case PRINT_SHEET_SELECTION:
-		compute_sheet_pages_add_sheet (pi,
-				     pi->sheet, TRUE, FALSE);
+		compute_sheet_pages_add_sheet (pi, pi->sheet, TRUE, FALSE);
 		break;
 	case PRINT_IGNORE_PRINTAREA:
-		compute_sheet_pages_add_sheet (pi,
-				     pi->sheet, FALSE, TRUE);
+		compute_sheet_pages_add_sheet (pi, pi->sheet, FALSE, TRUE);
 		break;
 	case PRINT_SHEET_SELECTION_IGNORE_PRINTAREA:
-		compute_sheet_pages_add_sheet (pi,
-				     pi->sheet, TRUE, TRUE);
+		compute_sheet_pages_add_sheet (pi, pi->sheet, TRUE, TRUE);
 		break;
 	}
 	return;
@@ -1257,6 +1097,7 @@ gnm_paginate_cb (GtkPrintOperation *operation,
 
 		gtk_print_operation_set_n_pages (operation, n_pages == 0 ? 1 : n_pages);
 		gtk_print_operation_set_unit (operation, GTK_UNIT_POINTS);
+		pi->hfi->pages = n_pages;
 
 		if (n_pages == 0) /* gtk+ cannot handle 0 pages */
 			gtk_print_operation_cancel (operation);
@@ -1291,7 +1132,7 @@ gnm_begin_print_cb (GtkPrintOperation *operation,
 	pr = gtk_print_settings_get_int_with_default
 		(settings, GNUMERIC_PRINT_SETTING_PRINTRANGE_KEY, PRINT_ACTIVE_SHEET);
 	if (from != pi->from || to != pi->to || pr != pi->pr) {
-		g_warning ("Working around gtk+ bug 423484.");
+		/* g_warning ("Working around gtk+ bug 423484."); */
 		gtk_print_settings_set_int
 			(settings, GNUMERIC_PRINT_SETTING_PRINT_FROM_SHEET_KEY,
 			 pi->from);
@@ -1348,7 +1189,7 @@ gnm_request_page_setup_cb (GtkPrintOperation *operation,
 
 	gsr = g_list_nth_data (pi->gnmSheetRanges, page_nr);
 	if (gsr == NULL) {
-		g_warning ("Avoiding gtk+ bug 492498");
+		/* g_warning ("Avoiding gtk+ bug 492498"); */
 		return;
 	}
 
@@ -1369,8 +1210,17 @@ gnm_draw_page_cb (GtkPrintOperation *operation,
 	PrintingInstance * pi = (PrintingInstance *) user_data;
 	SheetPageRange * gsr = g_list_nth_data (pi->gnmSheetRanges,
 					       page_nr);
-	if (gsr)
+	if (gsr) {
+		pi->hfi->page = page_nr + 1;
+		pi->hfi->sheet = gsr->sheet;
+		pi->hfi->page_area = gsr->range;
+		pi->hfi->top_repeating = gsr->range.start;
+		if (gsr->n_rep_cols > 0)
+			pi->hfi->top_repeating.col = gsr->first_rep_cols;
+		if (gsr->n_rep_rows > 0)
+			pi->hfi->top_repeating.row = gsr->first_rep_rows;
 		print_page (operation, context, pi, gsr);
+	}
 }
 
 static void
@@ -1640,8 +1490,21 @@ gnm_print_sheet (WorkbookControl *wbc, Sheet *sheet,
 
   res = gtk_print_operation_run (print, action, parent, NULL);
 
-  if (res == GTK_PRINT_OPERATION_RESULT_APPLY)
-	gnm_gconf_set_print_settings (gtk_print_operation_get_print_settings (print));
+  switch (res) {
+  case GTK_PRINT_OPERATION_RESULT_APPLY:
+	  gnm_gconf_set_print_settings (gtk_print_operation_get_print_settings (print));
+	  break;
+  case GTK_PRINT_OPERATION_RESULT_CANCEL:
+	  printing_instance_delete (pi);
+	  break;
+  case GTK_PRINT_OPERATION_RESULT_ERROR:
+	  /* FIXME? */
+	  break;
+  case GTK_PRINT_OPERATION_RESULT_IN_PROGRESS:
+	  /* FIXME? */
+	  break;
+  default: ;
+  }
 
   if (tmp_file_name) {
 	  char buffer[64 * 1024];
