@@ -16,6 +16,7 @@
 #include "sheet.h"
 #include "sheet-view.h"
 #include "sheet-merge.h"
+#include "sheet-style.h"
 #include "sheet-private.h"
 #include "sheet-control.h"
 #include "parse-util.h"
@@ -155,7 +156,7 @@ sv_is_colrow_selected (SheetView const *sv, int colrow, gboolean is_col)
  * @index :
  *
  * Returns TRUE if all of the selected cols/rows in the selection
- * 	are fully selected and the selection contains the specified col.
+ *	are fully selected and the selection contains the specified col.
  **/
 gboolean
 sv_is_full_colrow_selected (SheetView const *sv, gboolean is_cols, int index)
@@ -322,7 +323,7 @@ sv_menu_enable_insert (SheetView *sv, gboolean col, gboolean row)
 
 /**
  * selection_first_range
- * @sv        	 : The #SheetView whose selection we are testing.
+ * @sv		 : The #SheetView whose selection we are testing.
  * @wbc          : The calling context to report errors to (GUI or corba)
  * @command_name : A string naming the operation requiring a single range.
  *
@@ -554,9 +555,9 @@ set_menu_flags:
 	 *
 	 * WORKBOOK_FOREACH_VIEW (sheet->workbook, view, {
 	 *	if (sheet == wb_view_cur_sheet (view)) {
-	 *	        WORKBOOK_VIEW_FOREACH_CONTROL(view, wbc,
-	 *	                wb_control_menu_state_update (wbc, sheet, MS_SHOWHIDE_DETAIL););
- 	 *      }
+	 *		WORKBOOK_VIEW_FOREACH_CONTROL(view, wbc,
+	 *			wb_control_menu_state_update (wbc, sheet, MS_SHOWHIDE_DETAIL););
+	 *      }
 	 * });
 	 */
 }
@@ -575,7 +576,7 @@ sv_selection_set (SheetView *sv, GnmCellPos const *edit,
 
 /**
  * sv_selection_add_full :
- * @sheet          : sheet whose selection to append to.
+ * @sv             : #SheetView whose selection is append to.
  * @edit_{col,row} : cell to mark as the new edit cursor.
  * @base_{col,row} : stationary corner of the newly selected range.
  * @move_{col,row} : moving corner of the newly selected range.
@@ -617,10 +618,13 @@ sv_selection_add_pos (SheetView *sv, int col, int row)
 
 /**
  * sv_selection_free
- * @sv: the sheet
+ * @sv: #SheetView
  *
- * Releases the selection associated with this sheet
- */
+ * Releases the selection associated with @sv
+ *
+ * WARNING: This does not set a new selection and leaves the view in an
+ *		INVALID STATE.
+ **/
 void
 sv_selection_free (SheetView *sv)
 {
@@ -632,15 +636,16 @@ sv_selection_free (SheetView *sv)
 	sv->selections = NULL;
 }
 
-/*
+/**
  * sv_selection_reset :
  * @sv:  The sheet view
  *
- * Clears all of the selection ranges.
+ * Releases the selection associated with @sv , and forces a redraw of the
+ * previously selected regions and headers.
  *
  * WARNING: This does not set a new selection and leaves the view in an
- * 		INVALID STATE.
- */
+ *		INVALID STATE.
+ **/
 void
 sv_selection_reset (SheetView *sv)
 {
@@ -668,9 +673,11 @@ sv_selection_reset (SheetView *sv)
 
 /**
  * selection_get_ranges:
- * @sheet: the sheet.
+ * @sv: #SheetView
  * @allow_intersection : Divide the selection into nonoverlapping subranges.
- */
+ *
+ * Caller is responsible for free the list and the content.
+ **/
 GSList *
 selection_get_ranges (SheetView const *sv, gboolean allow_intersection)
 {
@@ -954,7 +961,7 @@ selection_get_ranges (SheetView const *sv, gboolean allow_intersection)
 
 /**
  * sv_selection_apply:
- * @sv: the sheet.
+ * @sv: #SheetView
  * @func:  The function to apply.
  * @allow_intersection : Call the routine for the non-intersecting subregions.
  * @closure : A parameter to pass to each invocation of @func.
@@ -1004,19 +1011,24 @@ typedef struct {
 static void
 cb_range_to_string (SheetView *sv, GnmRange const *r, void *closure)
 {
-	selection_to_string_closure * res = closure;
+	GnmConventionsOut out;
+	GnmRangeRef rr;
+	GnmParsePos pp;
+	selection_to_string_closure *res = closure;
 
-	if (*res->str->str)
+	if (res->str->len)
 		g_string_append_c (res->str, ',');
 
 	if (res->include_sheet_name_prefix)
 		g_string_append_printf (res->str, "%s!", sv->sheet->name_quoted);
 
-	g_string_append_printf (res->str, "$%s$%s",
-		col_name (r->start.col), row_name (r->start.row));
-	if ((r->start.col != r->end.col) || (r->start.row != r->end.row))
-		g_string_append_printf (res->str, ":$%s$%s",
-			col_name (r->end.col), row_name (r->end.row));
+	out.accum = res->str;
+	out.pp = parse_pos_init_sheet (&pp, sv->sheet);
+	out.convs = sheet_get_conventions (sv->sheet);
+
+	gnm_cellref_init (&rr.a, NULL, r->start.col, r->start.row, FALSE);
+	gnm_cellref_init (&rr.b, NULL, r->end.col, r->end.row, FALSE);
+	rangeref_as_string (&out, &rr);
 }
 
 char *
@@ -1065,6 +1077,22 @@ sv_selection_foreach (SheetView *sv,
 	return TRUE;
 }
 
+/* A protected sheet can limit whether locked and unlocked cells can be
+ * selected */
+gboolean
+sheet_selection_is_allowed (Sheet const *sheet, GnmCellPos const *pos)
+{
+	GnmStyle const *style;
+
+	if (!sheet->is_protected)
+		return TRUE;
+	style = sheet_style_get	(sheet, pos->col, pos->row);
+	if (gnm_style_get_contents_locked (style))
+		return sheet->protected_allow.select_locked_cells;
+	else
+		return sheet->protected_allow.select_unlocked_cells;
+}
+
 /**
  * walk_boundaries : Iterates through a region by row then column.
  *
@@ -1129,6 +1157,9 @@ loop :
 	if (cri != NULL && !cri->visible)
 		goto loop;
 
+	if (!sheet_selection_is_allowed (sv->sheet, &pos))
+		goto loop;
+
 	if (smart_merge) {
 		merge = gnm_sheet_merge_contains_pos (sv->sheet, &pos);
 		if (merge != NULL) {
@@ -1152,10 +1183,21 @@ loop :
 	return FALSE;
 }
 
+/**
+ * sv_selection_walk_step :
+ * @sv : #SheetView
+ * @forward :
+ * @horizontal :
+ *
+ * Move the edit_pos of @sv 1 step according to @forward and @horizontal.  The
+ * behavior depends several factors
+ *	- How many ranges are selected
+ *	- The shape of the selected ranges
+ *	- Previous movements (A sequence of tabs followed by an enter can jump
+ *		to the 1st col).
+ **/
 void
-sv_selection_walk_step (SheetView *sv,
-			gboolean forward,
-			gboolean horizontal)
+sv_selection_walk_step (SheetView *sv, gboolean forward, gboolean horizontal)
 {
 	int selections_count;
 	GnmCellPos destination;
@@ -1168,14 +1210,11 @@ sv_selection_walk_step (SheetView *sv,
 	ss = sv->selections->data;
 	selections_count = g_slist_length (sv->selections);
 
-	/* If there is no selection besides the cursor
-	 * iterate through the entire sheet.  Move the
-	 * cursor and selection as we go.
-	 * Ignore wrapping.  At that scale it is irrelevant.
-	 */
+	/* If there is no selection besides the cursor iterate through the
+	 * entire sheet.  Move the cursor and selection as we go.  Ignore
+	 * wrapping.  At that scale it is irrelevant.  */
 	if (selections_count == 1) {
-		if (ss->start.col == ss->end.col &&
-		    ss->start.row == ss->end.row)
+		if (range_is_singleton (ss))
 			is_singleton = TRUE;
 		else if (ss->start.col == sv->edit_pos.col &&
 			 ss->start.row == sv->edit_pos.row) {
@@ -1189,24 +1228,28 @@ sv_selection_walk_step (SheetView *sv,
 	if (is_singleton) {
 		int const first_tab_col = sv->first_tab_col;
 		int const cur_col = sv->edit_pos.col;
-		GnmRange full_sheet;
+		GnmRange bound;
 
-		if (horizontal) {
-			full_sheet.start.col = 0;
-			full_sheet.end.col   = SHEET_MAX_COLS-1;
-			full_sheet.start.row =
-				full_sheet.end.row   = ss->start.row;
-		} else {
-			full_sheet.start.col =
-				full_sheet.end.col   = ss->start.col;
-			full_sheet.start.row = 0;
-			full_sheet.end.row   = SHEET_MAX_ROWS-1;
-		}
+		/* Interesting :  Normally we bound the movement to the current
+		 *	col/row.  However, if a sheet is protected, and
+		 *	differentiates between selecting locked vs
+		 *	unlocked cells, then we do not bound things, and allow
+		 *	movement to any cell that is acceptable. */
+		if (sv->sheet->is_protected &&
+		    (sv->sheet->protected_allow.select_locked_cells ^
+		     sv->sheet->protected_allow.select_unlocked_cells))
+			range_init_full_sheet (&bound);
+		else if (horizontal)
+			range_init_rows (&bound, ss->start.row, ss->start.row);
+		else
+			range_init_cols (&bound, ss->start.col, ss->start.col);
 
 		/* Ignore attempts to move outside the boundary region */
-		if (!walk_boundaries (sv, &full_sheet, forward, horizontal,
+		if (!walk_boundaries (sv, &bound, forward, horizontal,
 				      FALSE, &destination)) {
-			if (!horizontal && first_tab_col >= 0)
+
+			/* <Enter> after some tabs jumps to the first col we tabbed from */
+			if (forward && !horizontal && first_tab_col >= 0)
 				destination.col = first_tab_col;
 
 			sv_selection_set (sv, &destination,
@@ -1215,7 +1258,7 @@ sv_selection_walk_step (SheetView *sv,
 			sv_make_cell_visible (sv, sv->edit_pos.col,
 					      sv->edit_pos.row, FALSE);
 			if (horizontal)
-				sv->first_tab_col = (first_tab_col < 0) ? cur_col : first_tab_col;
+				sv->first_tab_col = (first_tab_col < 0 || cur_col < first_tab_col) ? cur_col : first_tab_col;
 		}
 		return;
 	}
