@@ -34,6 +34,7 @@
 #include <goffice/utils/go-glib-extras.h>
 #include <glib/gi18n-lib.h>
 #include <string.h>
+#include <math.h>
 
 #ifndef USE_TILE_POOLS
 #define USE_TILE_POOLS 1
@@ -197,6 +198,8 @@ rstyle_apply (GnmStyle **old, ReplacementStyle *rs)
 #else
 #define TILE_SIZE_COL 32
 #endif
+#define PARTIAL_TILE_COL (SHEET_MAX_COLS != TILE_SIZE_COL * TILE_SIZE_COL * TILE_SIZE_COL * TILE_SIZE_COL)
+
 
 /* This is good until 16M rows.  */
 #if SHEET_MAX_ROWS <= 16 * 16 * 16 * 16
@@ -208,6 +211,7 @@ rstyle_apply (GnmStyle **old, ReplacementStyle *rs)
 #else
 #define	TILE_SIZE_ROW 64
 #endif
+#define PARTIAL_TILE_ROW (SHEET_MAX_ROWS != TILE_SIZE_ROW * TILE_SIZE_ROW * TILE_SIZE_ROW * TILE_SIZE_ROW)
 
 typedef enum {
 	TILE_UNDEFINED	= -1,
@@ -1070,7 +1074,7 @@ sheet_style_set_range (Sheet *sheet, GnmRange const *range,
  * @style : #GnmStyle
  *
  * NOTE : This is a simple wrapper for now.  When we support col/row styles it
- * 	will make life easier.
+ *	will make life easier.
  *
  * Change the complete style for a full col.
  * This function absorbs a reference to the new @style.
@@ -1089,7 +1093,7 @@ sheet_style_set_col (Sheet *sheet, int col, GnmStyle *style)
  * @pstyle : #GnmStyle
  *
  * NOTE : This is a simple wrapper for now.  When we support col/row styles it
- * 	will make life easier.
+ *	will make life easier.
  *
  * Apply a partial style to a full col.
  * The routine absorbs a reference to the partial style.
@@ -1108,7 +1112,7 @@ sheet_style_apply_col (Sheet *sheet, int col, GnmStyle *pstyle)
  * @style : #GnmStyle
  *
  * NOTE : This is a simple wrapper for now.  When we support col/row styles it
- * 	will make life easier.
+ *	will make life easier.
  *
  * Change the complete style for a full row.
  * This function absorbs a reference to the new @style.
@@ -1127,7 +1131,7 @@ sheet_style_set_row (Sheet  *sheet, int row, GnmStyle *style)
  * @pstyle : #GnmStyle
  *
  * NOTE : This is a simple wrapper for now.  When we support col/row styles it
- * 	will make life easier.
+ *	will make life easier.
  *
  * Apply a partial style to a full col.
  * The routine absorbs a reference to the partial style.
@@ -1147,7 +1151,7 @@ sheet_style_apply_row (Sheet  *sheet, int row, GnmStyle *pstyle)
  * @pstyle : #GnmStyle
  *
  * Apply a partial style to a single cell
- * This function absorbs a reference to the the new @style.
+ * This function absorbs a reference to the new @style.
  **/
 void
 sheet_style_apply_pos (Sheet *sheet, int col, int row,
@@ -1170,7 +1174,7 @@ sheet_style_apply_pos (Sheet *sheet, int col, int row,
  * @style :
  *
  * Change the complete style for a single cell.
- * This function absorbs a reference to the the new @style.
+ * This function absorbs a reference to the new @style.
  **/
 void
 sheet_style_set_pos (Sheet *sheet, int col, int row,
@@ -1406,7 +1410,7 @@ style_row_init (GnmBorder const * * *prev_vert,
 	 */
 	n = end_col - start_col + 3; /* 1 before, 1 after, 1 fencepost */
 	sr->vertical	 = mem;
-	sr->vertical 	-= start_col-1;
+	sr->vertical	-= start_col-1;
 	sr->top		 = sr->vertical + n;
 	sr->bottom	 = sr->top + n;
 	next_sr->top	 = sr->bottom; /* yes they should share */
@@ -1741,7 +1745,7 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 		sr.top [col] = none;
 
 	/* merge the bottom of the previous row */
- 	if (r->start.row > 0) {
+	if (r->start.row > 0) {
 		GnmBorder const ** roller;
 		sr.row = r->start.row - 1;
 		sheet_style_get_row (sheet, &sr);
@@ -1777,7 +1781,7 @@ sheet_style_find_conflicts (Sheet const *sheet, GnmRange const *r,
 	}
 
 	/* merge the top of the next row */
- 	if (r->end.row < (SHEET_MAX_ROWS-1)) {
+	if (r->end.row < (SHEET_MAX_ROWS-1)) {
 		sr.row = r->end.row + 1;
 		sheet_style_get_row (sheet, &sr);
 	}
@@ -2011,6 +2015,18 @@ cb_style_list_add_node (GnmStyle *style,
 	range.end.col = corner_col + width - 1;
 	range.end.row = corner_row + height - 1;
 
+#if PARTIAL_TILE_COL
+	if (corner_col >= SHEET_MAX_COLS)
+		return;
+	range.end.col = MIN (range.end.col, SHEET_MAX_COLS - 1);
+#endif
+
+#if PARTIAL_TILE_ROW
+	if (corner_row >= SHEET_MAX_ROWS)
+		return;
+	range.end.row = MIN (range.end.row, SHEET_MAX_ROWS - 1);
+#endif
+
 	if (apply_to) {
 		range.start.col -= apply_to->start.col;
 		if (range.start.col < 0)
