@@ -1,6 +1,6 @@
 /* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-#ifndef GNM_SHEET_H
-#define GNM_SHEET_H
+#ifndef _GNM_SHEET_H_
+# define _GNM_SHEET_H_
 
 #include "gnumeric.h"
 #include "colrow.h"
@@ -8,6 +8,8 @@
 #include <pango/pango.h>
 #include <goffice/utils/go-undo.h>
 
+G_BEGIN_DECLS
+
 typedef struct _SheetPrivate SheetPrivate;
 typedef enum {
 	GNM_SHEET_DATA,
@@ -15,11 +17,6 @@ typedef enum {
 	GNM_SHEET_XLM
 } GnmSheetType;
 
-typedef enum {
-	GNM_SHEET_VISIBILITY_VISIBLE,
-	GNM_SHEET_VISIBILITY_HIDDEN,
-	GNM_SHEET_VISIBILITY_VERY_HIDDEN
-} GnmSheetVisibility;
 GType gnm_sheet_visibility_get_type (void);
 #define GNM_SHEET_VISIBILITY_TYPE (gnm_sheet_visibility_get_type ())
 
@@ -357,4 +354,6 @@ do {										\
 	}										\
   } while (0)
 
-#endif /* GNM_SHEET_H */
+G_END_DECLS
+
+#endif /* _GNM_SHEET_H_ */
